package cn.com.infosec.jce.examples;

import cn.com.infosec.asn1.DERBMPString;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.asn1.x509.BasicConstraints;
import cn.com.infosec.asn1.x509.X509Extensions;
import cn.com.infosec.asn1.x509.X509Name;
import cn.com.infosec.jce.PrincipalUtil;
import cn.com.infosec.jce.X509Principal;
import cn.com.infosec.jce.interfaces.PKCS12BagAttributeCarrier;
import cn.com.infosec.jce.provider.InfosecProvider;
import cn.com.infosec.x509.X509V1CertificateGenerator;
import cn.com.infosec.x509.X509V3CertificateGenerator;
import cn.com.infosec.x509.extension.AuthorityKeyIdentifierStructure;
import cn.com.infosec.x509.extension.SubjectKeyIdentifierStructure;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Security;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.security.spec.RSAPrivateCrtKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

public class PKCS12Example {
   static char[] passwd = new char[]{'h', 'e', 'l', 'l', 'o', ' ', 'w', 'o', 'r', 'l', 'd'};
   static X509V1CertificateGenerator v1CertGen = new X509V1CertificateGenerator();
   static X509V3CertificateGenerator v3CertGen = new X509V3CertificateGenerator();

   public static Certificate createMasterCert(PublicKey pubKey, PrivateKey privKey) {
      v1CertGen.setSerialNumber(BigInteger.valueOf(1L));
      v1CertGen.setIssuerDN(new X509Principal("C=AU, O=The Legion of the Bouncy Castle, OU=Bouncy Primary Certificate"));
      v1CertGen.setNotBefore(new Date(System.currentTimeMillis() - 2592000000L));
      v1CertGen.setNotAfter(new Date(System.currentTimeMillis() + 2592000000L));
      v1CertGen.setSubjectDN(new X509Principal("C=AU, O=The Legion of the Bouncy Castle, OU=Bouncy Primary Certificate"));
      v1CertGen.setPublicKey(pubKey);
      v1CertGen.setSignatureAlgorithm("SHA1WithRSAEncryption");
      X509Certificate var10000 = v1CertGen.generateX509Certificate(privKey);
      var10000.checkValidity(new Date());
      var10000.verify(pubKey);
      ((PKCS12BagAttributeCarrier)var10000).setBagAttribute(PKCSObjectIdentifiers.pkcs_9_at_friendlyName, new DERBMPString("Bouncy Primary Certificate"));
      return var10000;
   }

   public static Certificate createIntermediateCert(PublicKey pubKey, PrivateKey caPrivKey, X509Certificate caCert) {
      Hashtable var3;
      var3 = new Hashtable.<init>();
      Vector var4;
      Vector var10000 = var4 = new Vector;
      var4.<init>();
      DERObjectIdentifier var5;
      var3.put(var5 = X509Name.C, "AU");
      DERObjectIdentifier var6;
      var3.put(var6 = X509Name.O, "The Legion of the Bouncy Castle");
      DERObjectIdentifier var7;
      var3.put(var7 = X509Name.OU, "Bouncy Intermediate Certificate");
      DERObjectIdentifier var8;
      var3.put(var8 = X509Name.EmailAddress, "feedback-crypto@Infosec.org");
      var10000.addElement(var5);
      var10000.addElement(var6);
      var10000.addElement(var7);
      var10000.addElement(var8);
      v3CertGen.reset();
      v3CertGen.setSerialNumber(BigInteger.valueOf(2L));
      v3CertGen.setIssuerDN(PrincipalUtil.getSubjectX509Principal(caCert));
      v3CertGen.setNotBefore(new Date(System.currentTimeMillis() - 2592000000L));
      v3CertGen.setNotAfter(new Date(System.currentTimeMillis() + 2592000000L));
      v3CertGen.setSubjectDN(new X509Principal(var4, var3));
      v3CertGen.setPublicKey(pubKey);
      v3CertGen.setSignatureAlgorithm("SHA1WithRSAEncryption");
      X509V3CertificateGenerator var12 = v3CertGen;
      DERObjectIdentifier var10001 = X509Extensions.SubjectKeyIdentifier;
      SubjectKeyIdentifierStructure var11;
      var11 = new SubjectKeyIdentifierStructure.<init>(pubKey);
      var12.addExtension(var10001, false, var11);
      var12 = v3CertGen;
      var10001 = X509Extensions.AuthorityKeyIdentifier;
      AuthorityKeyIdentifierStructure pubKey1;
      pubKey1 = new AuthorityKeyIdentifierStructure.<init>(caCert);
      var12.addExtension(var10001, false, pubKey1);
      var12 = v3CertGen;
      var10001 = X509Extensions.BasicConstraints;
      BasicConstraints pubKey2;
      pubKey2 = new BasicConstraints.<init>(0);
      var12.addExtension(var10001, true, pubKey2);
      X509Certificate var13 = v3CertGen.generateX509Certificate(caPrivKey);
      var13.checkValidity(new Date());
      var13.verify(caCert.getPublicKey());
      ((PKCS12BagAttributeCarrier)var13).setBagAttribute(PKCSObjectIdentifiers.pkcs_9_at_friendlyName, new DERBMPString("Bouncy Intermediate Certificate"));
      return var13;
   }

   public static Certificate createCert(PublicKey pubKey, PrivateKey caPrivKey, PublicKey caPubKey) {
      Hashtable var3;
      var3 = new Hashtable.<init>();
      Vector var4;
      Vector var10000 = var4 = new Vector;
      var4.<init>();
      DERObjectIdentifier var5;
      var3.put(var5 = X509Name.C, "AU");
      DERObjectIdentifier var6;
      var3.put(var6 = X509Name.O, "The Legion of the Bouncy Castle");
      DERObjectIdentifier var7;
      var3.put(var7 = X509Name.OU, "Bouncy Intermediate Certificate");
      DERObjectIdentifier var8;
      var3.put(var8 = X509Name.EmailAddress, "feedback-crypto@Infosec.org");
      var10000.addElement(var5);
      var10000.addElement(var6);
      var10000.addElement(var7);
      var10000.addElement(var8);
      Hashtable var12;
      var12 = new Hashtable.<init>();
      Vector var9;
      var10000 = var9 = new Vector;
      DERObjectIdentifier var10005 = var6;
      DERObjectIdentifier var10007 = var5;
      var9.<init>();
      var12.put(var5, "AU");
      var12.put(var6, "The Legion of the Bouncy Castle");
      var12.put(var5 = X509Name.L, "Melbourne");
      var12.put(var6 = X509Name.CN, "Eric H. Echidna");
      var12.put(var8, "feedback-crypto@Infosec.org");
      var9.addElement(var10007);
      var9.addElement(var10005);
      var9.addElement(var5);
      var9.addElement(var6);
      var10000.addElement(var8);
      v3CertGen.reset();
      v3CertGen.setSerialNumber(BigInteger.valueOf(3L));
      v3CertGen.setIssuerDN(new X509Principal(var4, var3));
      v3CertGen.setNotBefore(new Date(System.currentTimeMillis() - 2592000000L));
      v3CertGen.setNotAfter(new Date(System.currentTimeMillis() + 2592000000L));
      v3CertGen.setSubjectDN(new X509Principal(var9, var12));
      v3CertGen.setPublicKey(pubKey);
      v3CertGen.setSignatureAlgorithm("SHA1WithRSAEncryption");
      X509V3CertificateGenerator var13 = v3CertGen;
      DERObjectIdentifier var10001 = X509Extensions.SubjectKeyIdentifier;
      SubjectKeyIdentifierStructure var10;
      var10 = new SubjectKeyIdentifierStructure.<init>(pubKey);
      var13.addExtension(var10001, false, var10);
      var13 = v3CertGen;
      var10001 = X509Extensions.AuthorityKeyIdentifier;
      AuthorityKeyIdentifierStructure var11;
      var11 = new AuthorityKeyIdentifierStructure.<init>(caPubKey);
      var13.addExtension(var10001, false, var11);
      X509Certificate var14 = v3CertGen.generateX509Certificate(caPrivKey);
      var14.checkValidity(new Date());
      var14.verify(caPubKey);
      PKCS12BagAttributeCarrier var15 = (PKCS12BagAttributeCarrier)var14;
      ((PKCS12BagAttributeCarrier)var14).setBagAttribute(PKCSObjectIdentifiers.pkcs_9_at_friendlyName, new DERBMPString("Eric's Key"));
      var15.setBagAttribute(PKCSObjectIdentifiers.pkcs_9_at_localKeyId, new SubjectKeyIdentifierStructure(pubKey));
      return var14;
   }

   public static void main(String[] args) {
      Security.addProvider(new InfosecProvider());
      RSAPublicKeySpec args1;
      args1 = new RSAPublicKeySpec.<init>(new BigInteger("b4a7e46170574f16a97082b22be58b6a2a629798419be12872a4bdba626cfae9900f76abfb12139dce5de56564fab2b6543165a040c606887420e33d91ed7ed7", 16), new BigInteger("11", 16));
      RSAPrivateCrtKeySpec var1;
      RSAPrivateCrtKeySpec var10000 = var1 = new RSAPrivateCrtKeySpec;
      BigInteger var10001 = new BigInteger("b4a7e46170574f16a97082b22be58b6a2a629798419be12872a4bdba626cfae9900f76abfb12139dce5de56564fab2b6543165a040c606887420e33d91ed7ed7", 16);
      BigInteger var10002 = new BigInteger("11", 16);
      BigInteger var2;
      var2 = new BigInteger.<init>("9f66f6b05410cd503b2709e88115d55daced94d1a34d4e32bf824d0dde6028ae79c5f07b580f5dce240d7111f7ddb130a7945cd7d957d1920994da389f490c89", 16);
      BigInteger var3;
      var3 = new BigInteger.<init>("c0a0758cdf14256f78d4708c86becdead1b50ad4ad6c5c703e2168fbf37884cb", 16);
      BigInteger var4;
      var4 = new BigInteger.<init>("f01734d7960ea60070f1b06f2bb81bfac48ff192ae18451d5e56c734a5aab8a5", 16);
      BigInteger var5;
      var5 = new BigInteger.<init>("b54bb9edff22051d9ee60f9351a48591b6500a319429c069a3e335a1d6171391", 16);
      BigInteger var6;
      var6 = new BigInteger.<init>("d3d83daf2a0cecd3367ae6f8ae1aeb82e9ac2f816c6fc483533d8297dd7884cd", 16);
      BigInteger var7;
      var7 = new BigInteger.<init>("b8f52fc6f38593dabb661d3f50f8897f8106eee68b1bce78a95b132b4e5b5d19", 16);
      var10000.<init>(var10001, var10002, var2, var3, var4, var5, var6, var7);
      RSAPublicKeySpec var15;
      var15 = new RSAPublicKeySpec.<init>(new BigInteger("8de0d113c5e736969c8d2b047a243f8fe18edad64cde9e842d3669230ca486f7cfdde1f8eec54d1905fff04acc85e61093e180cadc6cea407f193d44bb0e9449b8dbb49784cd9e36260c39e06a947299978c6ed8300724e887198cfede20f3fbde658fa2bd078be946a392bd349f2b49c486e20c405588e306706c9017308e69", 16), new BigInteger("ffff", 16));
      RSAPrivateCrtKeySpec var17;
      var10000 = var17 = new RSAPrivateCrtKeySpec;
      var10001 = new BigInteger("8de0d113c5e736969c8d2b047a243f8fe18edad64cde9e842d3669230ca486f7cfdde1f8eec54d1905fff04acc85e61093e180cadc6cea407f193d44bb0e9449b8dbb49784cd9e36260c39e06a947299978c6ed8300724e887198cfede20f3fbde658fa2bd078be946a392bd349f2b49c486e20c405588e306706c9017308e69", 16);
      var10002 = new BigInteger("ffff", 16);
      var4 = new BigInteger.<init>("7deb1b194a85bcfd29cf871411468adbc987650903e3bacc8338c449ca7b32efd39ffc33bc84412fcd7df18d23ce9d7c25ea910b1ae9985373e0273b4dca7f2e0db3b7314056ac67fd277f8f89cf2fd73c34c6ca69f9ba477143d2b0e2445548aa0b4a8473095182631da46844c356f5e5c7522eb54b5a33f11d730ead9c0cff", 16);
      var5 = new BigInteger.<init>("ef4cede573cea47f83699b814de4302edb60eefe426c52e17bd7870ec7c6b7a24fe55282ebb73775f369157726fcfb988def2b40350bdca9e5b418340288f649", 16);
      var6 = new BigInteger.<init>("97c7737d1b9a0088c3c7b528539247fd2a1593e7e01cef18848755be82f4a45aa093276cb0cbf118cb41117540a78f3fc471ba5d69f0042274defc9161265721", 16);
      var7 = new BigInteger.<init>("6c641094e24d172728b8da3c2777e69adfd0839085be7e38c7c4a2dd00b1ae969f2ec9d23e7e37090fcd449a40af0ed463fe1c612d6810d6b4f58b7bfa31eb5f", 16);
      BigInteger var8;
      var8 = new BigInteger.<init>("70b7123e8e69dfa76feb1236d0a686144b00e9232ed52b73847e74ef3af71fb45ccb24261f40d27f98101e230cf27b977a5d5f1f15f6cf48d5cb1da2a3a3b87f", 16);
      BigInteger var9;
      var9 = new BigInteger.<init>("e38f5750d97e270996a286df2e653fd26c242106436f5bab0f4c7a9e654ce02665d5a281f2c412456f2d1fa26586ef04a9adac9004ca7f913162cb28e13bf40d", 16);
      var10000.<init>(var10001, var10002, var4, var5, var6, var7, var8, var9);
      RSAPublicKeySpec var19;
      var19 = new RSAPublicKeySpec.<init>(new BigInteger("b259d2d6e627a768c94be36164c2d9fc79d97aab9253140e5bf17751197731d6f7540d2509e7b9ffee0a70a6e26d56e92d2edd7f85aba85600b69089f35f6bdbf3c298e05842535d9f064e6b0391cb7d306e0a2d20c4dfb4e7b49a9640bdea26c10ad69c3f05007ce2513cee44cfe01998e62b6c3637d3fc0391079b26ee36d5", 16), new BigInteger("11", 16));
      RSAPrivateCrtKeySpec var21;
      var10000 = var21 = new RSAPrivateCrtKeySpec;
      var10001 = new BigInteger("b259d2d6e627a768c94be36164c2d9fc79d97aab9253140e5bf17751197731d6f7540d2509e7b9ffee0a70a6e26d56e92d2edd7f85aba85600b69089f35f6bdbf3c298e05842535d9f064e6b0391cb7d306e0a2d20c4dfb4e7b49a9640bdea26c10ad69c3f05007ce2513cee44cfe01998e62b6c3637d3fc0391079b26ee36d5", 16);
      var10002 = new BigInteger("11", 16);
      var6 = new BigInteger.<init>("92e08f83cc9920746989ca5034dcb384a094fb9c5a6288fcc4304424ab8f56388f72652d8fafc65a4b9020896f2cde297080f2a540e7b7ce5af0b3446e1258d1dd7f245cf54124b4c6e17da21b90a0ebd22605e6f45c9f136d7a13eaac1c0f7487de8bd6d924972408ebb58af71e76fd7b012a8d0e165f3ae2e5077a8648e619", 16);
      var7 = new BigInteger.<init>("f75e80839b9b9379f1cf1128f321639757dba514642c206bbbd99f9a4846208b3e93fbbe5e0527cc59b1d4b929d9555853004c7c8b30ee6a213c3d1bb7415d03", 16);
      var8 = new BigInteger.<init>("b892d9ebdbfc37e397256dd8a5d3123534d1f03726284743ddc6be3a709edb696fc40c7d902ed804c6eee730eee3d5b20bf6bd8d87a296813c87d3b3cc9d7947", 16);
      var9 = new BigInteger.<init>("1d1a2d3ca8e52068b3094d501c9a842fec37f54db16e9a67070a8b3f53cc03d4257ad252a1a640eadd603724d7bf3737914b544ae332eedf4f34436cac25ceb5", 16);
      BigInteger var10;
      var10 = new BigInteger.<init>("6c929e4e81672fef49d9c825163fec97c4b7ba7acb26c0824638ac22605d7201c94625770984f78a56e6e25904fe7db407099cad9b14588841b94f5ab498dded", 16);
      BigInteger var11;
      var11 = new BigInteger.<init>("dae7651ee69ad1d081ec5e7188ae126f6004ff39556bde90e0b870962fa7b926d070686d8244fe5a9aa709a95686a104614834b0ada4b10f53197a5cb4c97339", 16);
      var10000.<init>(var10001, var10002, var6, var7, var8, var9, var10, var11);
      KeyFactory var23;
      KeyFactory var25 = var23 = KeyFactory.getInstance("RSA", "INFOSEC");
      RSAPrivateCrtKeySpec var28 = var1;
      RSAPublicKeySpec var10003 = var15;
      PrivateKey var14 = var23.generatePrivate(var21);
      PublicKey var16 = var23.generatePublic(var19);
      PrivateKey var18 = var23.generatePrivate(var17);
      PublicKey var20 = var23.generatePublic(var10003);
      PrivateKey var22;
      PrivateKey var26 = var22 = var25.generatePrivate(var28);
      PublicKey args2 = var23.generatePublic(args1);
      Certificate[] var24;
      Certificate[] var30 = var24 = new Certificate[3];
      var24[2] = createMasterCert(var16, var14);
      var24[1] = createIntermediateCert(var20, var14, (X509Certificate)var24[2]);
      var30[0] = createCert(args2, var18, var20);
      PKCS12BagAttributeCarrier var27 = (PKCS12BagAttributeCarrier)var26;
      var27.setBagAttribute(PKCSObjectIdentifiers.pkcs_9_at_friendlyName, new DERBMPString("Eric's Key"));
      var27.setBagAttribute(PKCSObjectIdentifiers.pkcs_9_at_localKeyId, new SubjectKeyIdentifierStructure(args2));
      KeyStore var29 = KeyStore.getInstance("PKCS12", "INFOSEC");
      var29.load((InputStream)null, (char[])null);
      var29.setKeyEntry("Eric's Key", var22, (char[])null, var24);
      var29.store(new FileOutputStream("id.p12"), passwd);
   }
}
